LogMessage("running christmas_script.lua ")
require("main")

local height=100
local width=20

local brf=0.5 --- brightness factor

local targets={}
local targets_to_go=0

function AddTarget(pos,radius)
	table.insert(targets,{pos=pos,radius=radius})
	targets_to_go=targets_to_go+1
end

function Script:UpdateTargets()
	while true do
		Sleep(1)
		while targets_to_go do
			Sleep()
			for k,v in pairs(targets) do
				if v.pos then
					local a=SubVec(my_state.pos, v.pos)
					if a[1]*a[1]+a[2]*a[2]+a[3]*a[3] < v.radius*v.radius then
						targets[k]=nil
						targets_to_go=targets_to_go-1
					end
				end
			end
		end
	end
end

function Script:SpawnMeteorites()
--[[
	while true do
		Sleep(-math.log(SyncedRandom()+1E-9)*5.0)
		LogMessage("Spawning meteorites")
		NewGameObject{
			type="Meteorites",
			pos={100*(SyncedRandom()-0.5), 300*(SyncedRandom()-0.5),-150},
			vel={0,0,10}
		}
	end
	]]

	while true do
		Sleep(0.5)
		local p={RandomPoint()}
		NewGameObject{
			type="Meteorites",
			pos={p[1]-20, p[2], -100},
			vel={5,0,10},
			lifetime=100,
			mass=-math.log(SyncedRandom()+1E-9)*3.0
		}
	end
	--[[
	while true do
		Sleep(0.5)
		NewGameObject{
			type="Meteorites",
			pos={0, 0, -height*2},
			vel=ScaleVec(AddVec({RandomPoint()},{0,0,height}),0.05),
			lifetime=100,
			mass=-math.log(SyncedRandom()+1E-9)*3.0
		}
	end
	]]
end

function Script:SpawnRings()
	local h=0
	for i=0,60 do
		local a=h/height
		local w=width*(1-a)

		NewGameObject{type="AcceleratorSegment",
			orientation=AxisAngleToQuaternion({1,0,0},math.pi),
			use_coldet=false,

			pos={0,0,-h},
			scale=w*0.4,
			brightness=0.1*brf,
			speed=1,
			time_delta=i*0.2,
			health=0,
			radar_visible=false
		}
		h=h+w*0.2
		Sleep(0.3)
	end
end

local balls={}

function Script:UpdateBalls()
	while true do
		for i,id in ipairs(balls) do
			SetObjectState(id,{texture_offset=(music_y-1)/music_texture.height})
		end
		Sleep()
	end
end

function Script:SpawnFlowers()
	Sleep(3)
	local h=0
	local alpha=0
	local objs={}
	local nobjs=20
	for i=0,nobjs do
		alpha=alpha+math.pi*0.47
		local a=h/height
		local w=width*(1-a)

		local pos={math.sin(alpha)*w,math.cos(alpha)*w,-h}
		local r=w*0.2
		objs[i]={type="Flower",
			pos=pos,
			size=r,
			brightness=3*brf,
			quality=1,
			no_hud=1,
			radar_visible=false
		}
		AddTarget(pos, r)
		h=h+w*0.3
	end
	for i=0,nobjs do
		local obj=objs[nobjs-i]
		NewGameObject(obj)
		Sleep(0.4)
	end

	if music_texture then
		local h=0
		local alpha=0
		local objs={}
		local nobjs=10
		for i=0,nobjs do
			alpha=alpha+math.pi*0.32
			local a=h/height
			local w=width*(1-a)

			local pos={math.sin(alpha)*w,math.cos(alpha)*w,-h}
			local r=w*0.1

			local id=NewGameObject{
				type="MusicWaveSource",
				pos=pos,
				old_pos=pos,
				target=pos,
				radius=r,
				texture_id=music_texture.texture,
				texture_offset=(music_y-1)/music_texture.height,
				texture_scale=0.5,
				color={0.1*brf,0.1*brf,0.1*brf},
				color_phase=a*17.2,
				radar_visible=false
			}
			table.insert(balls,id)
			h=h+w*0.5
			Sleep(0.3)
		end
	end
end

override_sun_pos={0,0,100}
override_sun_radius=100
override_sun_color={0.07,0.07,0.07}

function Script:SpawnTree()
	NewGameObject{
	type="ChristmasTree",
	pos={0,0,0},
	orientation=AxisAngleToQuaternion({1,0,0},0),
	height=height,
	width=width*1.7,
	brightness=brf*1.5,
	}
	local pos={0,0,-height}
	NewGameObject{type="Flower", pos=pos, poly_n=4, size=7, seed_id=13, brightness=4*brf, no_hud=1, radar_visible=false}
	AddTarget(pos, 7)

	pos={0,0,-height+10}
	local id=NewGameObject{
				type="MusicWaveSource",
				pos=pos,
				old_pos=pos,
				target=pos,
				radius=2,
				texture_id=music_texture.texture,
				texture_offset=(music_y-1)/music_texture.height,
				texture_scale=0.5,
				color={0.05*brf,0.05*brf,0.05*brf},
				color_phase=2.3,
				radar_visible=false
			}
			table.insert(balls,id)
end


function Script:SpawnText()
	local t=5
	NewBillboard{
			lifetime=t,
			offset={-20,50,-200},
			align={1,0.5},
			text="Merry",
			scale=0.5
		}

		NewBillboard{
			lifetime=t,
			offset={20,50,-200},
			align={0,0.5},
			text="Christmas",
			scale=0.5
		}
end

function Script:SpawnStartBooster()
	local pos=my_state.pos
	NewGameObject{type="SimpleBonusPod", pos=pos, bonus_id=2, bonus_lifetime=2000}
end

function Script:SpawnSnowflakes()
	while true do
		local spawn_height=100
		local spin=3
		local speed=4
		NewGameObject{
			type="Snowflake",
			pos={(SyncedRandom()-0.5)*200, (SyncedRandom()-0.5)*200, -spawn_height},
			vel={0,0,speed},
			angular_vel={(SyncedRandom()-0.5)*spin,(SyncedRandom()-0.5)*spin,(SyncedRandom()-0.5)*spin},
			lifetime=spawn_height*1.2/speed,
			speed_factor=50,
			radar_visible=false,
			radius=1,
			collision_radius=2
		}
		Sleep(SyncedRandom()*0.1)
	end
end

--require("levels.special_snowflakes_spawn")

SetOption("misc/seen_christmas_tree", "1")
