local draw_spectra_to_texture=true

local use_32bit_float_texture=false
local use_nearest_filtering=false

local glver=gl.GetString("VERSION")
if glver<"3.3" then --- fix for geforce 7 series
	LogMessage("using fix for geforce 7 and earlier (opengl version <3.3) series GPUs.")
	use_32bit_float_texture=true
	use_nearest_filtering=true
end


function BellCurve(t)
	return math.exp(-t*t)
end

function FreqToColor(t)
	return {BellCurve((t-0.25)*6)+0.001,BellCurve((t-0.4)*6)+0.01,BellCurve((t-0.75)*6)}
end
--[[
function FreqToColor(t)
	return {BellCurve((t-0.75)*6),BellCurve((t-0.25)*6)+0.001,BellCurve((t-0.4)*6)+0.01}
end
]]

--- we have 48 notes per octave (4 notes per half-tone)
--- notes start 5 octaves below A440
local notes_per_octave=48

local max_freq=notes_per_octave*8

local rgb_t={0,0,0}
local sound_color={0,0,0}

local sound_color_raw={0,0,0}
local sound_color_avg={0.01,0.01,0.01}
local sound_color_avg_add={0.001,0.001,0.001}
local sound_color_avg_time=20

local mvis_brightness=0
local mvis_distortion=0
local mvis_speed=0
local mvis_filter_melody=0.5

notes_processed={}

if use_32bit_float_texture then
music_texture=TextureFramebuffer:new(64,1024,false,0x8814)
else
music_texture=TextureFramebuffer:new(64,1024)
end

music_texture.cleared=false
music_y=0
music_draw_y=0
sound_color_track_pos=0

local music_t=0
local music_vis_t=0
local music_vis_dt=1.0/70.0

local function DrawMusicToTexture(dt)
	local reserved_tracks=4

	music_t=music_t+dt
	local music_dy_exact=(music_t-music_vis_t)/music_vis_dt
	local music_dy=math.floor(music_dy_exact)
	if music_dy==0 then
		return
	end

	music_vis_t=music_vis_t+music_dy*music_vis_dt
	if music_dy>music_texture.height then
		music_dy=music_texture.height
	end

	local ns=math.floor(#notes_processed/music_texture.width)
	gl.UseProgram(0)
	music_texture:BindFB()
	if not music_texture.cleared then
		music_texture.cleared=true
		gl.Clear(gl.COLOR_BUFFER_BIT)
	end

	gl.PushAttrib(gl.VIEWPORT_BIT+gl.ENABLE_BIT)
	gl.Viewport(0, 0, music_texture.width, music_texture.height)
	gl.MatrixMode("PROJECTION")
	gl.PushMatrix()
	gl.LoadIdentity()
	gl.Ortho(0,music_texture.width,0,music_texture.height,-100,100);
	gl.MatrixMode("MODELVIEW")
	gl.PushMatrix()
	gl.LoadIdentity()

	gl.Disable("BLEND")
	gl.Disable("TEXTURE_2D")
	gl.Begin(gl.QUADS);
	local function DrawStrip(dy)
		local function DrawPx(i)
			gl.Vertex(i, music_draw_y)
			gl.Vertex(i, music_draw_y+dy)
			gl.Vertex(i+1, music_draw_y+dy)
			gl.Vertex(i+1, music_draw_y)
		end
		if draw_spectra_to_texture then
			for i=0,music_texture.width-1-reserved_tracks do
				local v1=0
				local v2=0
				for j=0,ns-1 do
					v1=v1+notes_processed[ns*i+j+1]
					v2=v2+notes[ns*i+j+1]
				end
				v1=v1/ns
				v1=v1*500
				v2=v2/ns
				v2=v2*500
				gl.Color(v1,v2,v1)
				DrawPx(i)
			end
		end
		sound_color_track_pos=(music_texture.width-1)/music_texture.width
		gl.Color(unpack(sound_color))
		DrawPx(music_texture.width-1)
		gl.Color(unpack(sound_color_raw))
		DrawPx(music_texture.width-2)
	end
	local next_music_y=music_draw_y+music_dy
	if next_music_y>music_texture.height then
		DrawStrip(music_texture.height-music_draw_y)
		music_draw_y=0
		next_music_y=next_music_y-music_texture.height
		DrawStrip(next_music_y)
		music_draw_y=next_music_y
	else
		DrawStrip(music_dy)
		music_draw_y=next_music_y
	end


	music_y=music_draw_y-1+music_dy_exact-music_dy;
	--music_y=music_draw_y

	gl.End()
	BindScreenFB()

	gl.MatrixMode("PROJECTION")
	gl.PopMatrix()

	gl.MatrixMode("MODELVIEW")
	gl.PopMatrix()

	gl.PopAttrib()
end

--sound_color={0.5,1,3}
--DrawMusicToTexture(0.2)

--- Here we override main.lua 's on_audio_frame. See data/main.lua for details
local main_on_audio_frame=on_audio_frame

local lag_protect_t=0
local lag_protect_nframes=0

function on_audio_frame(dt) --- called by engine

--- work around for GeForce 7 series bug, and similar
	if lag_protect_nframes>0 then
		lag_protect_t=lag_protect_t+dt
	end
	lag_protect_nframes=lag_protect_nframes+1
	if lag_protect_t>6 and lag_protect_nframes<30 then --- less than 10 fps in first 3 seconds, disable myself.
		LogMessage("Waves visualizer: aborting due to low FPS!")
		SetOption("game/music_visualizer", "2")
	end


	if dt>0.5 then
		dt=0.5
	end
	main_on_audio_frame(dt)
	if notes==nil then return end

	notes_processed={}
	notes_sum={}

	sound_color_raw={0,0,0}
	local sum=0

	for i,v in ipairs(notes) do
		if v<0 then
			v=0
		end
		sum=sum+v
		notes_sum[i]=sum
	end


	local mvis_filter_melody_b=1-mvis_filter_melody

	local window_r=notes_per_octave/6
	for i,v in ipairs(notes) do
		local a=(v-1.5*((notes_sum[i+window_r] or sum)-(notes_sum[i-window_r] or 0))*0.5/window_r)
		if a<0 then
			a=0
		end

		local q=(a*mvis_filter_melody + notes[i]*mvis_filter_melody_b)

		notes_processed[i]=q

		local f=i/max_freq
		local c=FreqToColor(f)

		q=q*2
		sound_color_raw[1]=sound_color_raw[1]+c[1]*q
		sound_color_raw[2]=sound_color_raw[2]+c[2]*q
		sound_color_raw[3]=sound_color_raw[3]+c[3]*q
	end
	--local color_v=(sound_color_raw[1]+sound_color_raw[2]+sound_color_raw[3])/3

	sound_color_avg=AddVec(ScaleVec(sound_color_avg, 1.0-dt/sound_color_avg_time), ScaleVec(sound_color_raw, dt/sound_color_avg_time))

	local a={1.0/(sound_color_avg[1]+sound_color_avg_add[1]), 1.0/(sound_color_avg[2]+sound_color_avg_add[2]), 1.0/(sound_color_avg[3]+sound_color_avg_add[3])}

	sound_color={sound_color_raw[1]*a[1], sound_color_raw[2]*a[2], sound_color_raw[3]*a[3] }
	for i=1,3 do
		if sound_color[i]>1 then
			sound_color_avg[i]=sound_color_avg[i]*sound_color[i]
		end
	end
	--[[
	local maxv=math.max(unpack(sound_color))
	if maxv>1 then
	  sound_color_avgv=sound_color_avgv*maxv
	end
	]]

	--LogMessage("sound color: ",table.tostring(sound_color))


	rgb_t[1]=rgb_t[1]+sound_color[1]*dt
	rgb_t[2]=rgb_t[2]+sound_color[2]*dt
	rgb_t[3]=rgb_t[3]+sound_color[3]*dt

	DrawMusicToTexture(dt)
end

function DrawHUDFunc.DrawMusicVisualizerDebug(width,height,bloom)-- called by engine, return true to suppress built-in drawer.
	if bloom then return false end

	if GetOption("devel/draw_debug_info")~="1" then return false end--- dont want this function now.

	if not notes then return end
	gl.Enable(gl.BLEND)
	gl.Begin(gl.QUADS);
	local piano_keyboard_height=20

	local black={false,true,false,true,false,false,true,false,true,false,true,false}

	local halftones=#notes*12/notes.steps_per_octave
	local halftone_width=notes.steps_per_octave/12

	local start_x=width-#notes

	local spectra_y=height
	local spectra_height=60
	local chart_y=height-piano_keyboard_height-spectra_height
	local piano_keyboard_y=height-spectra_height

	local function DrawBar(i,a)
		local x=i

		local ta=(math.log(a*1E5))*0.05
		if ta>0 then
			--- vertices of a rectangle are specified like this
			gl.Vertex(x+width-#notes, chart_y)
			gl.Vertex(x+width-#notes +1, chart_y)
			gl.Vertex(x+width-#notes +1, chart_y-ta*height)
			gl.Vertex(x+width-#notes, chart_y-ta*height)
		end
	end

	for i=2,#notes-1 do

		--local is_peak=a>0.52*(smooth_notes[i-1]+smooth_notes[i+1]) and a>smooth_avg_amp
		local x=i

		local f=i/max_freq
		local c=FreqToColor(f)

		local a=notes[i]
		gl.Color(0,1,0)
		DrawBar(i,a)

		a=notes_processed[i]
		gl.Color(1,0,0)
		DrawBar(i,a)

		a=a*8

		local c2={c[1]*a,c[2]*a,c[3]*a}

		gl.Color(unpack(c))
		gl.Vertex(x+width-#notes, spectra_y)
		gl.Vertex(x+width-#notes +1, spectra_y)
		gl.Color(unpack(c2))
		gl.Vertex(x+width-#notes +1, spectra_y-spectra_height*0.5)
		gl.Vertex(x+width-#notes, spectra_y-spectra_height*0.5)

		gl.Vertex(x+width-#notes, spectra_y-spectra_height*0.5)
		gl.Vertex(x+width-#notes +1, spectra_y-spectra_height*0.5)
		gl.Vertex(x+width-#notes +1, spectra_y-spectra_height)
		gl.Vertex(x+width-#notes, spectra_y-spectra_height)

	end
	--- draw octave markers
	local octaves=#notes/notes.steps_per_octave
	for i=0,octaves-1 do
		local x=i*notes.steps_per_octave+3*halftone_width
		gl.Color(0,0,0)
		gl.Vertex(x+width-#notes, piano_keyboard_y)
		gl.Vertex(x+width-#notes, piano_keyboard_y-piano_keyboard_height)
		gl.Color(0,0,0.5)
		gl.Vertex(x+width-#notes +notes.steps_per_octave, piano_keyboard_y - piano_keyboard_height)
		gl.Vertex(x+width-#notes +notes.steps_per_octave, piano_keyboard_y)
	end

	--- draw 'keyboard' underneath
	for i=0,halftones-1 do
		local is_black=black[((i-3)%12) +1]
		if is_black then
			gl.Color(0,0.1,0)
		else
			gl.Color(0,0.5,0)
		end
		local x=i*halftone_width
		gl.Vertex(x+width-#notes, piano_keyboard_y )
		gl.Vertex(x+width-#notes +halftone_width-1, piano_keyboard_y)
		gl.Vertex(x+width-#notes +halftone_width-1, piano_keyboard_y - piano_keyboard_height)
		gl.Vertex(x+width-#notes, piano_keyboard_y - piano_keyboard_height)
	end

	gl.End();



	gl.Enable("TEXTURE_2D")
	music_texture:BindTexture()
	gl.TexParameter("TEXTURE_2D","TEXTURE_WRAP_S","REPEAT")
	gl.TexParameter("TEXTURE_2D","TEXTURE_WRAP_T","REPEAT")
	gl.MatrixMode("TEXTURE")
	gl.LoadIdentity()
	gl.Translate(0,music_y/music_texture.height,0)
	gl.MatrixMode("MODELVIEW")
	gl.Begin("QUADS")
	gl.Color(1,1,1)
	gl.TexCoord(0, 0);
	gl.Vertex(0, 0)
	gl.TexCoord(0,1);
	gl.Vertex(music_texture.height, 0)
	gl.TexCoord(1,1);
	gl.Vertex(music_texture.height, music_texture.width)
	gl.TexCoord(1,0);
	gl.Vertex(0, music_texture.width)
	gl.End()

	gl.BindTexture("TEXTURE_2D", 0)

	gl.MatrixMode("TEXTURE")
	gl.LoadIdentity()
	gl.MatrixMode("MODELVIEW")

	return false -- do not override engine's DrawHUD
end

local centre={0,0,0}

local centre_obj_id=1

function Script:PlaceStart()
	if override_sun_pos then
		centre_obj_id=NewGameObject({
			type="MusicWaveSource",
			pos=override_sun_pos,
			old_pos=override_sun_pos,
			target=override_sun_pos,
			radius=override_sun_radius or 5
		})
	else
		centre=my_state.pos
		local pos={RandomPoint()}
		centre_obj_id=NewGameObject({
		type="MusicWaveSource",
		pos=pos,
		old_pos=pos,
		target={RandomPoint()},
		radius=override_sun_radius or 5
		})
		while true do
			Sleep(30)
			SetObjectState(centre_obj_id, {target={RandomPoint()}})
		end
	end
end


function load_scenefractal_shader(brightness, time, low, high, translate, scale)
	os.setlocale("C") -- fixing some weird bug
	mvis_brightness=GetOption("music_visualizer/brightness")+0
	mvis_distortion=GetOption("music_visualizer/distortion")+0
	mvis_speed=GetOption("music_visualizer/speed")+0
	mvis_blinkiness=GetOption("music_visualizer/blinkiness")+0
	mvis_filter_melody=GetOption("music_visualizer/filter_melody")+0

	if centre_obj_id~=1 then
		local data={draw_pos=true}
		GetObjectState(centre_obj_id, data)
		centre=data.draw_pos
		SetObjectState(centre_obj_id, {
		radius=override_sun_radius or 5,
		color=override_sun_color or {0.1,0.1,0.1},
		--color_phase=((glfwGetTime()*0.01) %1)*2*math.pi,
		color_phase=2.3,
		texture_id=music_texture.texture,
		texture_offset=(music_y-1)/music_texture.height,
		texture_scale=0.5,
		draw_HUD=true
		})
	end
	--LogMessage("low:" .. table.tostring(low))
	--LogMessage("high:" .. table.tostring(high))

	local sz=Len(SubVec(high, low))

	local radius=100/(2.03-mvis_speed)

	--local scale={1/(high[1]-low[1]),1/(high[2]-low[2]),1/(high[3]-low[3])}
	--local translate={-low[1]*scale[1],-low[2]*scale[2],-low[3]*scale[3]}
	--local translate={-(low[1]+high[1])*0.5*scale[1],-(low[2]+high[2])*0.5*scale[2],-(low[3]+high[3])*0.5*scale[3]}


	local shader=GetShader("$$data/music_visualizer_waves_1_v.glsl", "$$data/test_f.glsl")
	gl.UseProgram(shader)

	--{brightness*(0.2+sound_color[1]*pulse_strength), brightness*(0.2+sound_color[2]*pulse_strength), brightness*(0.2+sound_color[3]*pulse_strength)}

	music_texture:BindTexture()
	if use_nearest_filtering then
		gl.TexParameter("TEXTURE_2D", "TEXTURE_MIN_FILTER", "NEAREST");
		gl.TexParameter("TEXTURE_2D", "TEXTURE_MAG_FILTER", "NEAREST");
	end

	br=brightness*0.8

	gl.SetUniforms(shader, {
		brightness={br,br,br},
		stripe_brightness={
		10*mvis_brightness * (sound_color[1]*mvis_blinkiness+1.0-mvis_blinkiness),
		10*mvis_brightness * (sound_color[2]*mvis_blinkiness+1.0-mvis_blinkiness),
		10*mvis_brightness * (sound_color[3]*mvis_blinkiness+1.0-mvis_blinkiness)},
		music_offset=music_y/music_texture.height,
		track_pos=sound_color_track_pos,
		distort_scale=0.01*sz*mvis_distortion,
		stripe_matrix={
		scale/radius, 0, 0, 0,
		0, scale/radius, 0, 0,
		0, 0, scale/radius, 0,
		(translate[1]-centre[1])/radius, (translate[2]-centre[2])/radius, (translate[3]-centre[3])/radius, 1
		}
	})
	gl.MatrixMode("TEXTURE")
	gl.LoadIdentity()
	gl.MatrixMode("MODELVIEW")
end
